cat.analysis.fcn <- function(sites=NULL, subpop=NULL, design=NULL,
   data.cat=NULL, type.cat=NULL, N.cluster=NULL, popsize=NULL, stage1size=NULL,
   support=NULL, swgt=NULL, swgt1=NULL, unitsize=NULL, vartype="Local", conf=95,
   psurvey.obj=NULL) {


# Function:    cat.analysis.fcn
# Programmers: Tony Olsen
#              Tom Kincaid
# Date:        March 25, 2004
# Description:
#   This function organizes input and output for analysis of categorical data
#   generated by a probability survey.  Input can be either an object of class
#   "psurvey.analysis" (see the documentation for function psurvey.analysis.fcn)
#   or through use of the other arguments to this function.
#   Input:
#      sites = a data frame consisting of two variables: the first variable is 
#         site IDs, and the second variable is a logical vector indicating which
#         sites to use in the analysis.  If psurvey.obj is not provided, then
#         this argument is required.  The default is NULL.
#      subpop = a data frame describing sets of populations and subpopulations 
#         for which estimates will be calculated.  The first variable is site  
#         IDs.  Each subsequent variable identifies a Type of population, where
#         the variable name is used to identify Type.  A Type variable
#         identifies each site with one of the subpopulations of that Type.  If
#         psurvey.obj is not provided, then this argument is required.  The
#         default is NULL.
#      design = a data frame consisting of design variables.  If psurvey.obj is
#        not provided, then this argument is required.  The default is NULL.
#        Variables should be named as follows:
#            siteID = site IDs
#            wgt = final adjusted weights, which are either the weights for a
#               single-stage sample or the stage two weights for a two-stage
#               sample
#            xcoord = x-coordinates for location, which are either the x-
#               coordinates for a single-stage sample or the stage two x-
#               coordinates for a two-stage sample
#            ycoord = y-coordinates for location, which are either the y-
#               coordinates for a single-stage sample or the stage two y-
#               coordinates for a two-stage sample
#            stratum = the stratum codes
#            cluster = the stage one sampling unit (primary sampling unit or  
#               cluster) codes
#            wgt1 = final adjusted stage one weights
#            xcoord1 = the stage one x-coordinates for location
#            ycoord1 = the stage one y-coordinates for location
#      data.cat = a data frame of categorical response variables.  The first 
#         variable is site IDs.  Subsequent variables are response variables.
#         Missing data (NA) is allowed.  If psurvey.obj is not provided, then
#         this argument is required.  The default is NULL.
#      type.cat = the type of categorical response variables, where "Status" = 
#         site status and "Category" = resource category.  If this argument is
#         not provided, then each response variable is assigned type "Category".
#         The default is NULL.
#      N.cluster = the number of stage one sampling units in the resource, which 
#         is required for calculation of finite and continuous population 
#         correction factors for a two-stage sample.  For a stratified sample 
#         this variable must be a vector containing a value for each stratum and
#         must have the names attribute set to identify the stratum codes.  The
#         default is NULL.
#      popsize = the known size of the resource - the total number of sampling 
#         units of a discrete resource or the measure of a continuous resource,
#         which is required for calculation of finite and continuous population 
#         correction factors for a single-stage sample.  This variable is also 
#         used to adjust estimators for the known size of a resource.  For a
#         stratified sample this variable must be a vector containing a value 
#         for each stratum and must have the names attribute set to identify the
#         stratum codes.  The default is NULL.
#      stage1size = the known size of the stage one sampling units of a two-
#         stage sample, which is required for calculation of finite and  
#         continuous population correction factors for a two-stage sample and 
#         must have the names attribute set to identify the stage one sampling 
#         unit codes.  For a stratified sample, the names attribute must be set
#         to identify both stratum codes and stage one sampling unit codes using
#         a convention where the two codes are separated by the # symbol, e.g.,
#         "Stratum 1#Cluster 1".  The default is NULL.
#      support = the support value for each site - the value one (1) for a 
#         site from a discrete resource or the measure of the sampling unit  
#         associated with a site from a continuous resource, which is required  
#         for calculation of finite and continuous population correction  
#         factors.  The default is NULL.
#      swgt = the size-weight for each site, which is the stage two size-weight 
#         for a two-stage sample.  The default is NULL.
#      swgt1 = the stage one size-weight for each site.  The default is NULL.
#      unitsize = the known sum of the size-weights of the resource, which for a 
#         stratified sample must be a vector containing a value for each stratum 
#         and must have the names attribute set to identify the stratum codes.  
#         The default is NULL.
#      vartype = the choice of variance estimator, where "Local" = local mean
#         estimator and "SRS" = SRS estimator.  The default is "Local".
#      conf = the confidence level.  The default is 95%.
#      psurvey.obj = A list of class psurvey.analysis that was produced by the
#         function psurvey.analysis.fcn.  Depending on input to that function,
#         some elements of the list may be NULL.  The default is NULL.
#   Output:
#      A data frame of population estimates for all combinations of population 
#      Types, subpopulations within Types, response variables, and categories 
#      within each response variable.  Estimates are provided for proportion 
#      and size of the population plus standard error and confidence interval 
#      estimates.
#   Other Functions Required:
#      dframe.check.fcn - check site IDs, the sites data frame, the subpop data
#         frame, the data.cat data frame, type.cat, and the data.cont data frame
#         to assure valid contents and, as necessary, create the sites data frame,
#         the subpop data frame, and type.cat
#      input.check.fcn - check input values for compatibility
#      category.est.fcn - estimate proportion (expressed as percent) and size of a
#         resource in each of a set of categories
#   Examples:
#      mysiteID <- paste("Site", 1:100, sep="")
#      mysites <- data.frame(siteID=mysiteID, Active=rep(TRUE, 100))
#      mysubpop <- data.frame(siteID=mysiteID, Resource.Class=rep(c("A","B"),
#         c(55,45)))
#      mydesign <- data.frame(siteID=mysiteID, wgt=runif(100, 10, 100),
#         xcoord=runif(100), ycoord=runif(100), stratum=rep(c("Stratum1",
#         "Stratum2"), 50))
#      mydata.cat <- data.frame(siteID=mysiteID, CatVar=rep(c("north", "south",
#         "east", "west"), 25))
#      cat.analysis.fcn(sites=mysites, subpop=mysubpop, design=mydesign,
#         data.cat=mydata.cat)
#
#      mysites <- data.frame(siteID=mysiteID, Active=rep(c(TRUE, FALSE, TRUE,
#         TRUE), 25))
#      cat.analysis.fcn(sites=mysites, subpop=mysubpop, design=mydesign,
#         data.cat=mydata.cat)


# Begin the section when an object of class "psurvey.analysis" was input to the 
# function

   if(inherits(psurvey.obj, "psurvey.analysis")) {

# Assign variables from the input list

      sites <- psurvey.obj$sites
      subpop <- psurvey.obj$subpop
      design <- psurvey.obj$design
      data.cat <- psurvey.obj$data.cat
      type.cat <- psurvey.obj$type.cat
      N.cluster <- psurvey.obj$N.cluster
      popsize <- psurvey.obj$popsize
      stage1size <- psurvey.obj$stage1size
      support <- psurvey.obj$support
      swgt <- psurvey.obj$swgt
      swgt1 <- psurvey.obj$swgt1
      unitsize <- psurvey.obj$unitsize
      stratum.ind <- psurvey.obj$stratum.ind
      vartype <- psurvey.obj$vartype
      conf <- psurvey.obj$conf

# Begin the section when an object of class "psurvey.analysis" was not input to
# the function

   } else {

# Check that the required data frames have been provided

      if(is.null(sites))
         stop("\nThe sites data frame must be provided.")
      if(!is.data.frame(sites))
         stop("\nThe sites argument must be a data frame.")
      if(is.null(subpop))
         stop("\nThe subpop data frame must be provided.")
      if(!is.data.frame(subpop))
         stop("\nThe subpop argument must be a data frame.")
      if(is.null(design))
         stop("\nThe design data frame must be provided.")
      if(!is.data.frame(design))
         stop("\nThe design argument must be a data frame.")
      if(is.null(data.cat))
         stop("\nThe data.cat data frame must be provided.")
      if(!is.data.frame(data.cat))
         stop("\nThe data.cat argument must be a data frame.")

# Determine the number of response values

      nresp <- dim(design)[1]

# Check the design data frame for required names

      design.names <- names(design)
      temp <- match(design.names, c("siteID", "wgt", "xcoord", "ycoord", "stratum", "cluster", "wgt1", "xcoord1", "ycoord1"), nomatch=0)
      if(any(temp == 0))
         stop("\nThe names used in the design data frame do not match the required names.")

# Assign variables from the design data frame

      siteID <- design$siteID
      wgt <- design$wgt
      xcoord <- design$xcoord
      ycoord <- design$ycoord
      stratum <- design$stratum
      cluster <- design$cluster
      wgt1 <- design$wgt1
      xcoord1 <- design$xcoord1
      ycoord1 <- design$ycoord1

# Determine whether the sample is stratified

      stratum.ind <- length(unique(stratum)) > 1

# If the sample is stratified, convert stratum to a factor, determine stratum 
# levels, and calculate number of strata

      if(stratum.ind) {
         stratum <- factor(stratum)
         stratum.levels <- levels(stratum)
         nstrata <- length(stratum.levels)
      } else {
         stratum.levels <- NULL
         nstrata <- NULL
      }

# Determine whether the sample has two stages

      cluster.ind <- length(unique(cluster)) > 1

# If the sample has two stages, convert cluster to a factor, determine cluster 
# levels, and calculate number of clusters

      if(cluster.ind) {
         if(stratum.ind) {
            cluster.in <- cluster
            cluster <- tapply(cluster, stratum, factor)
            cluster.levels <- sapply(cluster, levels, simplify=FALSE)
            ncluster <- sapply(cluster.levels, length)
         } else {
            cluster <- factor(cluster)
            cluster.levels <- levels(cluster)
            ncluster <- length(cluster.levels)
         }
      }

# Determine whether the population correction factor is to be used

      pcfactor.ind <- !is.null(support)

# Determine whether the sample uses size-weights

      swgt.ind <- !is.null(swgt)

# Check the sites data frame, the subpop data frame, the data.cat data frame,
# and type.cat to assure valid contents

      temp <- dframe.check.fcn(nresp, siteID, sites, subpop, data.cat,
         type.cat, NULL, design.names)
      sites <- temp$sites
      subpop <- temp$subpop
      data.cat <- temp$data.cat
      type.cat <- temp$type.cat

# Check for compatibility of input values

      temp <- input.check.fcn(nresp, wgt, NULL, NULL, xcoord, ycoord,
         stratum.ind, stratum, stratum.levels, nstrata, cluster.ind, cluster,
         cluster.levels, ncluster, N.cluster, wgt1, xcoord1, ycoord1, popsize,
         stage1size, pcfactor.ind, support, swgt.ind, swgt, swgt1, unitsize,
         vartype, conf)

      N.cluster <- temp$N.cluster
      popsize <- temp$popsize
      stage1size <- temp$stage1size
      unitsize <- temp$unitsize

# If the sample was stratified and had two stages, then reset cluster to its 
# input value

      if(stratum.ind && cluster.ind)
         cluster <- cluster.in

# As necessary, assign missing values to the design variables

      if(is.null(xcoord))
         xcoord <- rep(NA, nresp)
      if(is.null(ycoord))
         ycoord <- rep(NA, nresp)
      if(is.null(stratum))
         stratum <- rep(NA, nresp)
      if(is.null(cluster))
         cluster <- rep(NA, nresp)
      if(is.null(wgt1))
         wgt1 <- rep(NA, nresp)
      if(is.null(xcoord1))
         xcoord1 <- rep(NA, nresp)
      if(is.null(ycoord1))
         ycoord1 <- rep(NA, nresp)

# Recreate the design data frame

      design <- data.frame(siteID=siteID, wgt=wgt, xcoord=xcoord, ycoord=ycoord,
         stratum=stratum, cluster=cluster, wgt1=wgt1, xcoord1=xcoord1,
         ycoord1=ycoord1)

# As nesessary, remove rows of the design, sites, subpop, and data.cat, data
# frames based on the logical variable in the sites data frame

      subpop <- subpop[sites[,2],]
      design <- design[sites[,2],]
      data.cat <- data.cat[sites[,2],]
      sites <- sites[sites[,2],]

# End the section when an object of class "psurvey.analysis" was not input to
# the function

   }

# Loop through all response variables

   nvar <- dim(data.cat)[2]
   varnames <- names(data.cat)
   ntypes <- dim(subpop)[2]
   typenames <- names(subpop)
   nrow <- 0 
   for(ivar in 2:nvar) {

# Loop through all types of populations

      for(itype in 2:ntypes){

# Find unique subpopulations of this type of population

         subpopnames <- levels(factor(subpop[,itype]))	

# Loop through all subpopulations of this type

         for(isubpop in 1:length(subpopnames)) {

# Select sites in a subpopulation

            subpop.ind <- subpop[,itype] == subpopnames[isubpop]

# For a stratified sample, remove values from N.cluster, popsize, stage1size, 
# and unitsize for strata that do not occur in the response variable

            if(stratum.ind) {
               temp.N.cluster <- N.cluster[!is.na(match(names(N.cluster),
                  unique(design[subpop.ind, 5])))]
               temp.popsize <- popsize[!is.na(match(names(popsize),
                  unique(design[subpop.ind, 5])))]
               temp.stage1size <- stage1size[!is.na(match(names(stage1size),
                  unique(design[subpop.ind, 5])))]
               temp.unitsize <- unitsize[!is.na(match(names(unitsize),
                  unique(design[subpop.ind, 5])))]
            } else {
               temp.N.cluster <- N.cluster
               temp.popsize <- popsize
               temp.stage1size <- stage1size
               temp.unitsize <- unitsize
            }

# Estimate category proportions for the response variable

            temp.cat <- category.est.fcn(catvar=data.cat[subpop.ind,ivar],
               wgt=design[subpop.ind,2], x=design[subpop.ind,3],
               y=design[subpop.ind,4], stratum=design[subpop.ind,5],
               cluster=design[subpop.ind,6], N.cluster=temp.N.cluster,
               wgt1=design[subpop.ind,7], x1=design[subpop.ind,8],
               y1=design[subpop.ind,9], popsize=temp.popsize,
               stage1size=temp.stage1size, support=support[subpop.ind],
               swgt=swgt[subpop.ind], swgt1=swgt1[subpop.ind],
               unitsize=temp.unitsize, type=type.cat[ivar-1], vartype=vartype,
               conf=conf, check.ind=FALSE)

# Assign category proportion estimates for the response variable to a data frame

            if(nrow == 0) {
               nn <- dim(temp.cat)[1]
               catsum <- data.frame(Type=rep(typenames[itype],nn), 
                  Subpopulation=rep(subpopnames[isubpop],nn), 
                  Indicator=rep(varnames[ivar],nn), temp.cat)
               nrow <- nn
            } else {
               nn <- dim(temp.cat)[1]
               catsum <- rbind(catsum, data.frame(Type=rep(typenames[itype],nn), 
                  Subpopulation=rep(subpopnames[isubpop],nn), 
                  Indicator=rep(varnames[ivar],nn), temp.cat, 
                  row.names=(nrow+1):(nrow+nn)))
               nrow <- nrow + nn
            }

# End of the loop for subpopulations

         }

# End of the loop for type of population

      }

# End of the loop for response variables

   }


# Return the data frame

   catsum
}
