cont.analysis.fcn <- function(sites=NULL, subpop=NULL, design=NULL,
   data.cont=NULL, sigma=NULL, var.sigma=NULL, N.cluster=NULL, popsize=NULL,
   stage1size=NULL, support=NULL, swgt=NULL, swgt1=NULL, unitsize=NULL,
   vartype="Local", conf=95, pctval=c(5, 25, 50, 75, 95), psurvey.obj=NULL) {


# Function:    cont.analysis.fcn
# Programmers: Tony Olsen
#              Tom Kincaid
# Date:        March 16, 2004
# Description:
#   This function organizes input and output for analysis of continuous data
#   generated by a probability survey.  Input can be either an object of class
#   "psurvey.analysis" (see the documentation for function psurvey.analysis.fcn)
#   or through use of the other arguments to this function.
#   Input:
#      sites = a data frame consisting of two variables: the first variable is 
#         site IDs, and the second variable is a logical vector indicating which
#         sites to use in the analysis.  If psurvey.obj is not provided, then
#         this argument is required.  The default is NULL.
#      subpop = a data frame describing sets of populations and subpopulations 
#         for which estimates will be calculated.  The first variable is site  
#         IDs.  Each subsequent variable identifies a Type of population, where
#         the variable name is used to identify Type.  A Type variable
#         identifies each site with one of the subpopulations of that Type.  If
#         psurvey.obj is not provided, then this argument is required.  The
#         default is NULL.
#      design = a data frame consisting of design variables.  If psurvey.obj is
#        not provided, then this argument is required.  The default is NULL.
#        Variables should be named as follows:
#            siteID = site IDs
#            wgt = final adjusted weights, which are either the weights for a
#               single-stage sample or the stage two weights for a two-stage
#               sample
#            xcoord = x-coordinates for location, which are either the x-
#               coordinates for a single-stage sample or the stage two x-
#               coordinates for a two-stage sample
#            ycoord = y-coordinates for location, which are either the y-
#               coordinates for a single-stage sample or the stage two y-
#               coordinates for a two-stage sample
#            stratum = the stratum codes
#            cluster = the stage one sampling unit (primary sampling unit or  
#               cluster) codes
#            wgt1 = final adjusted stage one weights
#            xcoord1 = the stage one x-coordinates for location
#            ycoord1 = the stage one y-coordinates for location
#      data.cont = a data frame of continuous response variables.  The first 
#         variable is site IDs.  Subsequent variables are response variables.
#         Missing data (NA) is allowed.  The default is NULL.
#      sigma = measurement error variance.  This variable must be a vector
#         containing a value for each response variable and must have the names
#         attribute set to identify the response variable names.  Missing data
#         (NA) is allowed.  The default is NULL.
#      var.sigma = variance of the measurement error variance.  This variable
#         must be a vector containing a value for each response variable and
#         must have the names attribute set to identify the response variable
#         names.  Missing data (NA) is allowed.  The default is NULL.
#      N.cluster = the number of stage one sampling units in the resource, which 
#         is required for calculation of finite and continuous population 
#         correction factors for a two-stage sample.  For a stratified sample 
#         this variable must be a vector containing a value for each stratum and
#         must have the names attribute set to identify the stratum codes.  The
#         default is NULL.
#      popsize = the known size of the resource - the total number of sampling 
#         units of a discrete resource or the measure of a continuous resource,
#         which is required for calculation of finite and continuous population 
#         correction factors for a single-stage sample.  This variable is also 
#         used to adjust estimators for the known size of a resource.  For a
#         stratified sample this variable must be a vector containing a value 
#         for each stratum and must have the names attribute set to identify the
#         stratum codes.  The default is NULL.
#      stage1size = the known size of the stage one sampling units of a two-
#         stage sample, which is required for calculation of finite and  
#         continuous population correction factors for a two-stage sample and 
#         must have the names attribute set to identify the stage one sampling 
#         unit codes.  For a stratified sample, the names attribute must be set
#         to identify both stratum codes and stage one sampling unit codes using
#         a convention where the two codes are separated by the # symbol, e.g.,
#         "Stratum 1#Cluster 1".  The default is NULL.
#      support = the support value for each site - the value one (1) for a 
#         site from a discrete resource or the measure of the sampling unit  
#         associated with a site from a continuous resource, which is required  
#         for calculation of finite and continuous population correction  
#         factors.  The default is NULL.
#      swgt = the size-weight for each site, which is the stage two size-weight 
#         for a two-stage sample.  The default is NULL.
#      swgt1 = the stage one size-weight for each site.  The default is NULL.
#      unitsize = the known sum of the size-weights of the resource, which for a 
#         stratified sample must be a vector containing a value for each stratum 
#         and must have the names attribute set to identify the stratum codes.  
#         The default is NULL.
#      vartype = the choice of variance estimator, where "Local" = local mean
#         estimator and "SRS" = SRS estimator.  The default is "Local".
#      conf = the confidence level.  The default is 95%.
#      pctval = the set of values at which percentiles are estimated.  The 
#         default set is: {5, 25, 50, 75, 95}.
#      psurvey.obj = A list of class psurvey.analysis that was produced by the
#         function psurvey.analysis.fcn.  Depending on input to that function,
#         some elements of the list may be NULL.  The default is NULL.
#   Output:
#      A list containing either three or five data frames of population
#      estimates for all combinations of population Types, subpopulations within
#      Types, and response variables.  The data frames containing deconvoluted
#      CDF estimates and deconvoluted percentile estimates are only included in
#      the output list when an input value for measurement error variance is
#      provided to the function.  CDF and percentile estimates are calculated
#      for both proportion and size of the population.  Standard error estimates
#      and confidence interval estimates also are calculated.  
#      The five data frames are:
#         CDF - a data frame containing the CDF estimates
#         Pct - a data frame containing the percentile estimates
#         CDF.D - a data frame containing the deconvoluted CDF estimates
#         Pct.D - a data frame containing the deconvoluted percentile estimates
#         Tot - a data frame containing the total, mean, standard deviation,
#            and variance estimates
#      If an input value for measurement error variance is not provided to the
#      function, then CDF.D and Pct.D are assigned the value NULL.
#   Other Functions Required:
#      dframe.check.fcn - check site IDs, the sites data frame, the subpop data
#         frame, the data.cat data frame, type.cat, and the data.cont data frame
#         to assure valid contents and, as necessary, create the sites data
#         frame, the subpop data frame, and type.cat
#      input.check.fcn - check input values for compatibility
#      cdf.est.fcn - estimate the cumulative distribution function (CDF) for the 
#         proportion (expressed as percent) and the total of a response variable
#      cdf.decon.fcn - estimate the deconvoluted CDF for the proportion and the
#         total of a response variable
#      total.est.fcn - estimate the population total, mean, variance, and
#         standard deviation of a response variable
#   Examples:
#      mysiteID <- paste("Site", 1:25, sep="")
#      mysites <- data.frame(siteID=mysiteID, Active=rep(TRUE, 25))
#      mysubpop <- data.frame(siteID=mysiteID, All.Sites=rep("All Sites",25))
#      mydesign <- data.frame(siteID=mysiteID, wgt=runif(25, 10, 100),
#         xcoord=runif(25), ycoord=runif(25))
#      ContVar <- rnorm(25, 10, 1)
#      mydata.cont <- data.frame(siteID=mysiteID, ContVar=ContVar)
#      cont.analysis.fcn(sites=mysites, subpop=mysubpop, design=mydesign,
#         data.cont=mydata.cont)
#
#      mydata.cont <- data.frame(siteID=mysiteID, ContVar=ContVar,
#         ContVar.1=ContVar + rnorm(25, 0, sqrt(0.25)),
#         ContVar.2=ContVar + rnorm(25, 0, sqrt(0.50)))
#      mysigma <- c(NA, 0.25, 0.50)
#      names(mysigma) <- c("ContVar", "ContVar.1", "ContVar.2")
#      cont.analysis.fcn(sites=mysites, subpop=mysubpop, design=mydesign,
#         data.cont=mydata.cont, sigma=mysigma)


# Begin the section when an object of class "psurvey.analysis" was input to the 
# function

   if(inherits(psurvey.obj, "psurvey.analysis")) {

# Assign variables from the input list

      sites <- psurvey.obj$sites
      subpop <- psurvey.obj$subpop
      design <- psurvey.obj$design
      data.cont <- psurvey.obj$data.cont
      sigma <- psurvey.obj$sigma
      var.sigma <- psurvey.obj$var.sigma
      N.cluster <- psurvey.obj$N.cluster
      popsize <- psurvey.obj$popsize
      stage1size <- psurvey.obj$stage1size
      support <- psurvey.obj$support
      swgt <- psurvey.obj$swgt
      swgt1 <- psurvey.obj$swgt1
      unitsize <- psurvey.obj$unitsize
      stratum.ind <- psurvey.obj$stratum.ind
      cluster.ind <- psurvey.obj$cluster.ind
      pcfactor.ind <- psurvey.obj$pcfactor.ind
      swgt.ind.ind <- psurvey.obj$swgt.ind.ind
      vartype <- psurvey.obj$vartype
      conf <- psurvey.obj$conf
      pctval <- psurvey.obj$pctval

# Begin the section when an object of class "psurvey.analysis" was not input to
# the function

   } else {

# Check that the required data frames have been provided

      if(is.null(sites))
         stop("\nThe sites data frame must be provided.")
      if(!is.data.frame(sites))
         stop("\nThe sites argument must be a data frame.")
      if(is.null(subpop))
         stop("\nThe subpop data frame must be provided.")
      if(!is.data.frame(subpop))
         stop("\nThe subpop argument must be a data frame.")
      if(is.null(design))
         stop("\nThe design data frame must be provided.")
      if(!is.data.frame(design))
         stop("\nThe design argument must be a data frame.")
      if(is.null(data.cont))
         stop("\nThe data.cont data frame must be provided.")
      if(!is.data.frame(data.cont))
         stop("\nThe data.cont argument must be a data frame.")

# Determine the number of response values

      nresp <- dim(design)[1]

# Check the design data frame for required names

      design.names <- names(design)
      temp <- match(design.names, c("siteID", "wgt", "xcoord", "ycoord", "stratum", "cluster", "wgt1", "xcoord1", "ycoord1"), nomatch=0)
      if(any(temp == 0))
         stop("\nThe names used in the design data frame do not match the required names.")

# Assign variables from the design data frame

      siteID <- design$siteID
      wgt <- design$wgt
      xcoord <- design$xcoord
      ycoord <- design$ycoord
      stratum <- design$stratum
      cluster <- design$cluster
      wgt1 <- design$wgt1
      xcoord1 <- design$xcoord1
      ycoord1 <- design$ycoord1

# Determine whether the sample is stratified

      stratum.ind <- length(unique(stratum)) > 1

# If the sample is stratified, convert stratum to a factor, determine stratum 
# levels, and calculate number of strata

      if(stratum.ind) {
         stratum <- factor(stratum)
         stratum.levels <- levels(stratum)
         nstrata <- length(stratum.levels)
      } else {
         stratum.levels <- NULL
         nstrata <- NULL
      }

# Determine whether the sample has two stages

      cluster.ind <- length(unique(cluster)) > 1

# If the sample has two stages, convert cluster to a factor, determine cluster 
# levels, and calculate number of clusters

      if(cluster.ind) {
         if(stratum.ind) {
            cluster.in <- cluster
            cluster <- tapply(cluster, stratum, factor)
            cluster.levels <- sapply(cluster, levels, simplify=FALSE)
            ncluster <- sapply(cluster.levels, length)
         } else {
            cluster <- factor(cluster)
            cluster.levels <- levels(cluster)
            ncluster <- length(cluster.levels)
         }
      }

# Determine whether the population correction factor is to be used

      pcfactor.ind <- !is.null(support)

# Determine whether the sample uses size-weights

      swgt.ind <- !is.null(swgt)

# Check the sites data frame, the subpop data frame, and the data.cont data
# frame to assure valid contents

      temp <- dframe.check.fcn(nresp, siteID, sites, subpop, NULL, NULL,
         data.cont, design.names)
      sites <- temp$sites
      subpop <- temp$subpop
      data.cont <- temp$data.cont

# Check the vector of measurement error variance values for correct names

      if(!is.null(sigma)) {
         temp.names <- names(data.cont)[-1]
         if(length(sigma) != length(temp.names))
            stop("\nThe vector of measurement error variance values is not the correct length.")
         if(is.null(names(sigma)))
            stop("\nThe vector of measurement error variance values must be named.")
         temp <- match(temp.names, names(sigma), nomatch=0)
         if(any(temp == 0)) {
            temp.str <- vecprint.fcn(temp.names[temp == 0])
            stop(paste("\nThe following names for the response variables do not occur among the names for \nthe vector of measurement error variance values:\n", temp.str, "\n", sep=""))
         }
         temp <- match(names(sigma), temp.names, nomatch=0)
         if(any(temp == 0)) {
            temp.str <- vecprint.fcn(names(sigma)[temp == 0])
            stop(paste("\nThe following names for the vector of measurement error variance values do not \noccur among the names for the response variables:\n", temp.str, "\n", sep=""))
         }
         sigma <- sigma[temp]
      }

# Check the vector of values for variance of the measurement error variance for
# correct names

      if(!is.null(var.sigma)) {
         if(length(var.sigma) != length(temp.names))
            stop("\nThe vector of values for variance of the measurement error variance is not the \ncorrect length.")
         if(is.null(names(var.sigma)))
            stop("\nThe vector of values for variance of the measurement error variance must be \nnamed.")
         temp <- match(temp.names, names(var.sigma), nomatch=0)
         if(any(temp == 0)) {
            temp.str <- vecprint.fcn(temp.names[temp == 0])
            stop(paste("\nThe following names for the response variables do not occur among the names for \nthe vector of values for variance of the measurement error variance:\n", temp.str, "\n", sep=""))
         }
         temp <- match(names(var.sigma), temp.names, nomatch=0)
         if(any(temp == 0)) {
            temp.str <- vecprint.fcn(names(var.sigma)[temp == 0])
            stop(paste("\nThe following names for the vector of values for variance of the measurement \nerror variance do not occur among the names for the response variables:\n", temp.str, "\n", sep=""))
         }
         var.sigma <- var.sigma[temp]
      }

# Check for compatibility of input values

      temp <- input.check.fcn(nresp, wgt, sigma, var.sigma, xcoord, ycoord,
         stratum.ind, stratum, stratum.levels, nstrata, cluster.ind, cluster,
         cluster.levels, ncluster, N.cluster, wgt1, xcoord1, ycoord1, popsize,
         stage1size, pcfactor.ind, support, swgt.ind, swgt, swgt1, unitsize,
         vartype, conf, pctval=pctval)

      N.cluster <- temp$N.cluster
      popsize <- temp$popsize
      stage1size <- temp$stage1size
      unitsize <- temp$unitsize

# If the sample was stratified and had two stages, then reset cluster to its 
# input value

      if(stratum.ind && cluster.ind)
         cluster <- cluster.in

# As necessary, assign missing values to the design variables

      if(is.null(xcoord))
         xcoord <- rep(NA, nresp)
      if(is.null(ycoord))
         ycoord <- rep(NA, nresp)
      if(is.null(stratum))
         stratum <- rep(NA, nresp)
      if(is.null(cluster))
         cluster <- rep(NA, nresp)
      if(is.null(wgt1))
         wgt1 <- rep(NA, nresp)
      if(is.null(xcoord1))
         xcoord1 <- rep(NA, nresp)
      if(is.null(ycoord1))
         ycoord1 <- rep(NA, nresp)

# Recreate the design data frame

      design <- data.frame(siteID=siteID, wgt=wgt, xcoord=xcoord, ycoord=ycoord,
         stratum=stratum, cluster=cluster, wgt1=wgt1, xcoord1=xcoord1,
         ycoord1=ycoord1)

# As nesessary, remove rows of the design, sites, subpop, and data.cont, data
# frames based on the logical variable in the sites data frame

      subpop <- subpop[sites[,2],]
      design <- design[sites[,2],]
      data.cont <- data.cont[sites[,2],]
      sites <- sites[sites[,2],]

# End the section when an object of class "psurvey.analysis" was not input to
# the function

   }

# Loop through all response variables

   nvar <- dim(data.cont)[2]
   varname <- names(data.cont)
   ntypes <- dim(subpop)[2]
   typenames <- names(subpop)
   nrow1a <- 0 
   nrow2a <- 0 
   for (ivar in 2:nvar) {

# Find unique data values across all subpopulations of this type of population

      cdfval <- sort(unique(data.cont[!is.na(data.cont[,ivar]),ivar]))

# Loop through all types of populations

      for (itype in 2:ntypes){

# Find unique subpopulations of this type of population

         subpopnames <- levels(factor(subpop[,itype]))	

# Loop through all subpopulations of this type

         for (isubpop in 1:length(subpopnames)) {

# Select sites in a subpopulation

            subpop.ind <- subpop[,itype] == subpopnames[isubpop]

# For a stratified sample, remove values from N.cluster, popsize, stage1size, 
# and unitsize for strata that do not occur in the response variable

            if(stratum.ind) {
               temp.N.cluster <- N.cluster[!is.na(match(names(N.cluster),
                  unique(design[subpop.ind, 5])))]
               temp.popsize <- popsize[!is.na(match(names(popsize),
                  unique(design[subpop.ind, 5])))]
               temp.stage1size <- stage1size[!is.na(match(names(stage1size),
                  unique(design[subpop.ind, 5])))]
               temp.unitsize <- unitsize[!is.na(match(names(unitsize),
                  unique(design[subpop.ind, 5])))]
            } else {
               temp.N.cluster <- N.cluster
               temp.popsize <- popsize
               temp.stage1size <- stage1size
               temp.unitsize <- unitsize
            }

# Calculate estimates for the response variable

            temp1.cont <- cdf.est.fcn(z=data.cont[subpop.ind,ivar],
               wgt=design[subpop.ind,2], x=design[subpop.ind,3],
               y=design[subpop.ind,4], stratum=design[subpop.ind,5],
               cluster=design[subpop.ind,6], N.cluster=temp.N.cluster,
               wgt1=design[subpop.ind,7], x1=design[subpop.ind,8],
               y1=design[subpop.ind,9], popsize=temp.popsize,
               stage1size=temp.stage1size, support=support[subpop.ind],
               swgt=swgt[subpop.ind], swgt1=swgt1[subpop.ind],
               unitsize=temp.unitsize, vartype=vartype, conf=conf,
               cdfval=cdfval, pctval=pctval, check.ind=FALSE)

            if(!is.null(sigma) && !is.na(sigma[ivar-1]))
               temp2.cont <- cdf.decon.fcn(z=data.cont[subpop.ind,ivar],
                  wgt=design[subpop.ind,2], sigma=sigma[ivar-1],
                  var.sigma=var.sigma[ivar-1], x=design[subpop.ind,3],
                  y=design[subpop.ind,4], stratum=design[subpop.ind,5],
                  cluster=design[subpop.ind,6], N.cluster=temp.N.cluster,
                  wgt1=design[subpop.ind,7], x1=design[subpop.ind,8],
                  y1=design[subpop.ind,9], popsize=temp.popsize,
                  stage1size=temp.stage1size, support=support[subpop.ind],
                  swgt=swgt[subpop.ind], swgt1=swgt1[subpop.ind],
                  unitsize=temp.unitsize, vartype=vartype, conf=conf,
                  cdfval=cdfval, pctval=pctval, check.ind=FALSE)

            temp3.cont <- total.est.fcn(z=data.cont[subpop.ind,ivar],
               wgt=design[subpop.ind,2], x=design[subpop.ind,3],
               y=design[subpop.ind,4], stratum=design[subpop.ind,5],
               cluster=design[subpop.ind,6], N.cluster=temp.N.cluster,
               wgt1=design[subpop.ind,7], x1=design[subpop.ind,8],
               y1=design[subpop.ind,9], popsize=temp.popsize,
               stage1size=temp.stage1size, support=support[subpop.ind],
               swgt=swgt[subpop.ind], swgt1=swgt1[subpop.ind],
               unitsize=temp.unitsize, vartype=vartype, conf=conf,
               check.ind=FALSE)

# Assign estimates for the response variable to data frames

            if(nrow1a == 0) {
               nn <- dim(temp1.cont$CDF)[1]
               cdfsum <- data.frame(Type=rep(typenames[itype],nn), 
                  Subpopulation=rep(subpopnames[isubpop],nn), 
                  Indicator=rep(varname[ivar],nn), temp1.cont$CDF)
               nrow1a <- nn

               nn <- dim(temp1.cont$Pct)[1]
               pctsum <- data.frame(Type=rep(typenames[itype],nn), 
                  Subpopulation=rep(subpopnames[isubpop],nn), 
                  Indicator=rep(varname[ivar],nn), temp1.cont$Pct)
               nrow1b <- nn

               nn <- dim(temp3.cont)[1]
               totsum <- data.frame(Type=rep(typenames[itype],nn), 
                  Subpopulation=rep(subpopnames[isubpop],nn), 
                  Indicator=rep(varname[ivar],nn), temp3.cont)
               nrow3 <- nn

            } else {
               nn <- dim(temp1.cont$CDF)[1]
               cdfsum <- rbind(cdfsum, data.frame(Type=rep(typenames[itype],nn), 
                  Subpopulation=rep(subpopnames[isubpop],nn), 
                  Indicator=rep(varname[ivar],nn), temp1.cont$CDF, 
                  row.names=(nrow1a+1):(nrow1a+nn)))
               nrow1a <- nrow1a + nn

               nn <- dim(temp1.cont$Pct)[1]
               pctsum <- rbind(pctsum, data.frame(Type=rep(typenames[itype],
                  nn), Subpopulation=rep(subpopnames[isubpop],nn),
                  Indicator= rep(varname[ivar],nn), temp1.cont$Pct, 
                  row.names=(nrow1b+1):(nrow1b+nn)))
               nrow1b <- nrow1b + nn

               nn <- dim(temp3.cont)[1]
               totsum <- rbind(totsum, data.frame(Type=rep(typenames[itype],nn), 
                  Subpopulation=rep(subpopnames[isubpop],nn), 
                  Indicator=rep(varname[ivar],nn), temp3.cont, 
                  row.names=(nrow3+1):(nrow3+nn)))
               nrow3 <- nrow3 + nn
            }

            if(!is.null(sigma) && !is.na(sigma[ivar-1])) {
               if(nrow2a == 0) {
                  nn <- dim(temp2.cont$CDF)[1]
                  cdfsum.D <- data.frame(Type=rep(typenames[itype],nn), 
                     Subpopulation=rep(subpopnames[isubpop],nn), 
                     Indicator=rep(varname[ivar],nn), temp2.cont$CDF)
                  nrow2a <- nn

                  nn <- dim(temp2.cont$Pct)[1]
                  pctsum.D <- data.frame(Type=rep(typenames[itype],nn), 
                     Subpopulation=rep(subpopnames[isubpop],nn), 
                     Indicator=rep(varname[ivar],nn), temp2.cont$Pct)
                  nrow2b <- nn
               } else {
                  nn <- dim(temp2.cont$CDF)[1]
                  cdfsum.D <- rbind(cdfsum.D, data.frame(Type=rep(typenames
                     [itype], nn), Subpopulation=rep(subpopnames[isubpop],nn), 
                     Indicator=rep(varname[ivar],nn), temp2.cont$CDF, 
                     row.names=(nrow2a+1):(nrow2a+nn)))
                  nrow2a <- nrow2a + nn

                  nn <- dim(temp2.cont$Pct)[1]
                  pctsum.D <- rbind(pctsum.D, data.frame(Type=rep(typenames
                     [itype],nn), Subpopulation=rep(subpopnames[isubpop],nn), 
                     Indicator=rep(varname[ivar],nn), temp2.cont$Pct, 
                     row.names=(nrow2b+1):(nrow2b+nn)))
                  nrow2b <- nrow2b + nn
               }
            }

# End of the loop for subpopulations

         }

# End of the loop for type of population

      }

# End of the loop for response variables

   }

# Return the list

   if(!is.null(sigma))
      list(CDF=cdfsum, Pct=pctsum, CDF.D=cdfsum.D, Pct.D=pctsum.D, Tot=totsum)
   else
      list(CDF=cdfsum, Pct=pctsum, CDF.D=NULL, Pct.D=NULL, Tot=totsum)
}
